package gregapi.item;

import static gregapi.data.CS.*;
import gregapi.GT_API;
import gregapi.api.Abstract_Mod;
import gregapi.data.LH;
import gregapi.data.MD;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictMaterialStack;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidContainerItem;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public class ItemFluidDisplay extends Item implements IFluidContainerItem, IItemUpdatable {
	protected IIcon mIcon;
	private final String mName;
	
	public ItemFluidDisplay() {
		super();
		mName = "gt.display.fluid";
		LH.add(mName + ".name", "Fluid Display");
		GameRegistry.registerItem(this, mName, MD.GAPI.mID);
		if (!D1) UT.Stacks.hide(this);
	}
	
	@Override
	public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		Fluid tFluid = FluidRegistry.getFluid(UT.Stacks.meta(aStack));
		if (D1) {
			if (tFluid != null) aList.add("Registry: " + tFluid.getName());
		}
		if (aNBT != null) {
			long tToolTipAmount = aNBT.getLong("a");
			if (tToolTipAmount > 0) {
				aList.add(EnumChatFormatting.BLUE + "Amount: " + tToolTipAmount + " L" + EnumChatFormatting.GRAY);
				if (tFluid != null) {
					OreDictMaterialStack tMaterial = OreDictMaterial.FLUID_MAP.get(tFluid.getName());
					if (tMaterial != null && tMaterial.mAmount > 0) {
						long tAmount = UT.Code.units(tToolTipAmount, tMaterial.mAmount, U, F);
						if (tAmount > 0) {
							int tDigits = (int)(((tAmount % U) / UD) * 1000);
							aList.add(EnumChatFormatting.BLUE + "Worth: " + (tAmount / U) + "." + (tDigits<1?"000":tDigits<10?"00"+tDigits:tDigits<100?"0"+tDigits:tDigits) + " Units of " + tMaterial.mMaterial.getLocal() + EnumChatFormatting.GRAY);
						}
					}
				}
			}
			aList.add(EnumChatFormatting.RED + "Temperature: " + aNBT.getLong("h") + " K" + EnumChatFormatting.GRAY);
			aList.add(EnumChatFormatting.GREEN + "State: " + (aNBT.getBoolean("s")?"Gas":"Liquid") + EnumChatFormatting.GRAY);
		}
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void registerIcons(IIconRegister aIconRegister) {
		// Useful hack to register Block Icons. That is why the Fluid Display Item has to exist always.
		if (Abstract_Mod.sFinalized >= Abstract_Mod.sModCountUsingGTAPI) {
			OUT.println("GT_API: Setting up Icon Register for Blocks");
			GT_API.sBlockIcons = aIconRegister;
			OUT.println("GT_API: Starting Block Icon Load Phase");
			for (Runnable tRunnable : GT_API.sBlockIconload) {
				try {
					tRunnable.run();
				} catch(Throwable e) {
					e.printStackTrace(ERR);
				}
			}
			OUT.println("GT_API: Registering Crop specific Textures");
			try {
				for (gregapi.old.GT_BaseCrop tCrop : gregapi.old.GT_BaseCrop.sCropList) tCrop.registerSprites(aIconRegister);
			} catch(Throwable e) {
				e.printStackTrace(ERR);
			}
			OUT.println("GT_API: Finished Block Icon Load Phase");
		}
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public IIcon getIconFromDamage(int aMeta) {
		Fluid tFluid = FluidRegistry.getFluid(aMeta);
		return tFluid==null?FluidRegistry.WATER.getStillIcon():tFluid.getStillIcon();
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public int getColorFromItemStack(ItemStack aStack, int aRenderPass) {
		Fluid tFluid = FluidRegistry.getFluid(UT.Stacks.meta(aStack));
		return tFluid==null?16777215:tFluid.getColor();
	}
	
	@Override
	public int getSpriteNumber() {
		return 0;
	}
	
	@Override
	public String getUnlocalizedName(ItemStack aStack) {
		if (aStack != null) return UT.Fluids.name(FluidRegistry.getFluid(UT.Stacks.meta(aStack)), F);
		return "";
	}
	
	@Override
	public String getItemStackDisplayName(ItemStack aStack) {
		if (aStack != null) return UT.Fluids.name(FluidRegistry.getFluid(UT.Stacks.meta(aStack)), T);
		return "";
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(Item aItem, CreativeTabs aTab, List aList) {
		if (D1) for (int i = 0, j = FluidRegistry.getMaxID(); i <= j; i++) {
			ItemStack tStack = UT.Fluids.display(FluidRegistry.getFluid(i));
			if (tStack != null) aList.add(tStack);
		}
		for (String tName : UT.Books.BOOK_LIST) aList.add(UT.Books.getBookWithTitle(tName));
	}
	
	@Override public final Item setUnlocalizedName(String aName) {return this;}
	@Override public final String getUnlocalizedName() {return mName;}
	
	@Override
	public boolean doesSneakBypassUse(World aWorld, int aX, int aY, int aZ, EntityPlayer aPlayer) {
		return T;
	}
	
	@Override
	public ItemStack getContainerItem(ItemStack aStack) {
		return null;
	}
	
	@Override
	public final boolean hasContainerItem(ItemStack aStack) {
		return F;
	}
	
	@Override
	public void updateItemStack(ItemStack aStack) {
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT == null) aStack.setTagCompound(aNBT = new NBTTagCompound());
		Fluid tFluid = UT.Fluids.fluid(aNBT.getString("f"));
		if (tFluid == null) {
			tFluid = UT.Fluids.fluid(UT.Stacks.meta(aStack));
			if (tFluid == null) {
				UT.Stacks.meta(aStack, W);
			} else {
				aNBT.setString("f", tFluid.getName());
			}
		} else {
			UT.Stacks.meta(aStack, tFluid.getID());
		}
	}
	
	@Override
	public FluidStack getFluid(ItemStack aStack) {
		Fluid tFluid = FluidRegistry.getFluid(UT.Stacks.meta(aStack));
		if (tFluid == null) return null;
		FluidStack rFluid = null;
		NBTTagCompound aNBT = aStack.getTagCompound();
		if (aNBT != null) {
			long tAmount = aNBT.getLong("a");
			if (tAmount > 0) rFluid = UT.Fluids.make(tFluid, tAmount);
		}
		return rFluid == null ? UT.Fluids.make(tFluid, 0) : rFluid;
	}
	
	@Override
	public int getCapacity(ItemStack aStack) {
		return Integer.MAX_VALUE;
	}
	
	@Override
	public int fill(ItemStack aStack, FluidStack aFluid, boolean aDoFill) {
		return 0;
	}
	
	@Override
	public FluidStack drain(ItemStack aStack, int aDrain, boolean aDoDrain) {
		return getFluid(aStack);
	}
}